from twisted.internet import reactor
from coherence.base import Coherence
from service import CAS, Tuner, AVTransport, FDC, ConnectionManager
from cetoncore import success, error, info, warning

class Instance:
    def __init__(self, device):
        self.device = device
        self.cas = None
        self.tuner = None
        self.av_transport = None
        self.fdc = None
        self.cm = None

        for s in self.device.get_services():
            id = s.get_id()
            if id == Tuner.id:
                self.tuner = Tuner(s)
            elif id == CAS.id:
                self.cas = CAS(s)
            elif id == AVTransport.id:
                self.av_transport = AVTransport(s)
            elif id == FDC.id:
                self.fdc = FDC(s)
            elif id == ConnectionManager.id:
                self.cm = ConnectionManager(s)


class Board:
    def __init__(self, interface, ready):
        self.device = None
        self.interface = interface
        self.ready = ready
        self.instances = []
        self.init_coherence()

    def board_search(self):
        for d in self.coherence.get_devices():
            self.check_device(d)
        if self.device is None:
            reactor.callLater(1, self.board_search)

    def check_device(self, device):
        if self.device is None:
            if 'InfiniTV' in device.get_friendly_name():
                info("Found: %s" % device.get_friendly_name() )
                self.device = device
            if self.device is not None:
                reactor.callLater(1, self.setup_device)

    def get_instance(self, instance):
        return self.instances[instance]

    def setup_device(self):
        for d in self.device.get_devices():
            self.instances.append(Instance(d))
        self.ready(self)

    def device_found(self, device):
        self.check_device(device)

    def device_removed(self, usn):
        if self.device.get_usn() == usn:
            self.device = None

    def init_coherence(self):
        self.coherence = Coherence({'logging':
            { 'level': 'debug',
                'subsystem': [
                    {'name':'action', 'active':'yes', 'level': 'none' },
                    {'name':'msearch', 'active':'yes', 'level': 'none' },
                    {'name':'device', 'active':'yes', 'level': 'none' },
                    {'name':'coherence', 'active':'yes', 'level': 'none' },
                    {'name':'ssdp', 'active':'yes', 'level': 'none' },
                    {'name':'soap', 'active':'yes', 'level': 'none' },
                    {'name':'variable', 'active':'yes', 'level': 'none' },
                    {'name':'event', 'active':'yes', 'level': 'none' },
                    {'name':'event_protocol', 'active':'yes', 'level': 'none' },
                    {'name':'event_server', 'active':'yes', 'level': 'none' },
                    ],
                #'logfile': 'client_upnp.log',
                }
            })
        self.coherence.add_interface(self.interface)
        self.coherence.connect(self.device_found, 'Coherence.UPnP.RootDevice.detection_completed')
        self.coherence.connect(self.device_removed, 'Coherence.UPnP.RootDevice.removed')
        self.coherence.init_control_point()
        for device in self.coherence.devices:
            self.device_found(device)

