from cetoncore import success, error, info, warning
import cetoncore
from twisted.internet import reactor, protocol, defer
from twisted.protocols.basic import LineReceiver

class RTPClient(protocol.DatagramProtocol):
    def __init__(self, data_callback):
        self.data_callback = data_callback

    def datagramReceived(self, datagram, address):
        self.data_callback(datagram)

class RTSPClientProtocol(LineReceiver):
    def __init__(self, deferred, client):
        self._deferred = deferred
        self.client = client
        self.response_lines = []

    def connectionMade(self):
        self.sendLine("SETUP rtsp://%s:%d/%s RTSP/1.0" % (self.client.server_ip, self.client.server_port, self.client.track))
        self.sendLine("CSeq: %d" % (self.client.cseq))
        self.sendLine("Blocksize: %d" % (self.client.block_size))
        self.sendLine("Transport: RTP/AVP;unicast;client_port=%d-%d" % (self.client.rtp_port_start, self.client.rtp_port_end))
        self.sendLine("")

    def lineReceived(self, line):
        self.response_lines.append(line)
        if len(line) == 0:
            if self._deferred is None:
                return
            d, self._deferred = self._deferred , None
            d.callback(self.response_lines)
            self.response_lines = []


class RTSPClient:
    def __init__(self, transport_uri, data_callback, instance):
        self.transport_uri = transport_uri
        if self.transport_uri.startswith("rtsp://"):
            self.transport_uri = self.transport_uri[len("rtsp://"):]
        comp = self.transport_uri.split(":")
        self.server_ip = comp[0]
        comp2 = comp[1].split("/")
        self.server_port = int(comp2[0])
        self.track = comp2[1]
        self.rtp_port_start = 5000 + (2*instance)
        self.rtp_port_end = 5001 + (2*instance)
        self.cseq = 135047189
        self.block_size = 524288

        self.data_callback = data_callback

    def setup(self):

        reactor.listenUDP(self.rtp_port_start, RTPClient(self.data_callback))

        d = defer.Deferred()
        client_creator = protocol.ClientCreator( reactor, RTSPClientProtocol, d, self)
        client_creator.connectTCP(self.server_ip, self.server_port)
        return d

    def play(self):
        pass
