from cetoncore import success, error, info, warning

class Service:
    def __init__(self, service):
        self.service = service
        self.service.subscribe()
        self.service.client = self

    def subscribe_for_variable(self, var, callback):
        self.service.subscribe_for_variable(var, callback=callback)

    def get_state_variable(self, var):
        return self.service.get_state_variable(var)
        

class CAS(Service):
    id = "urn:opencable-com:serviceId:urn:schemas-opencable-com:service:CAS"
    def __init__(self, service):
        Service.__init__(self, service)
        self.service = service


    def SetChannel(self,
            NewCaptureMode, NewChannelNumber, NewSourceId):
        action = self.service.get_action("SetChannel")
        return action.call(NewCaptureMode=NewCaptureMode,
                NewChannelNumber=NewChannelNumber,
                NewSourceId=NewSourceId)
    

class AVTransport(Service):
    id = "urn:upnp-org:serviceId:urn:schemas-upnp-org:service:AVTransport"
    def __init__(self, service):
        Service.__init__(self, service)
        self.instance_id = 0

    def set_instance_id(self, instance_id):
        self.instance_id = int(instance_id)
    
    def Play(self):
        action = self.service.get_action("Play")
        return action.call(InstanceID=self.instance_id, Speed=1)

class ConnectionManager(Service):
    id = "urn:upnp-org:serviceId:urn:schemas-upnp-org:service:ConnectionManager"
    def __init__(self, service):
        Service.__init__(self, service)

    def PrepareForConnection(self):
        action = self.service.get_action("PrepareForConnection")
        return action.call(RemoteProtocolInfo="", PeerConnectionManager="",
                PeerConnectionID="", Direction="Output")


class FDC(Service):
    id = "urn:opencable-com:serviceId:urn:schemas-opencable-com:service:FDC"
    def __init__(self, service):
        Service.__init__(self, service)


class Tuner(Service):
    id = "urn:opencable-com:serviceId:urn:schemas-opencable-com:service:Tuner"
    def __init__(self, service):
        Service.__init__(self, service)

    def SeekSignal(self,
            StartFrequency,
            StopFrequency,
            NewModulationList,
            Increment,
            SeekUp,
            TimeToBlock):
        action = self.service.get_action("SeekSignal")
        return action.call(StartFrequency=StartFrequency,
                StopFrequency=StopFrequency,
                NewModulationList=NewModulationList,
                Increment=Increment,
                SeekUp=str(SeekUp),
                TimeToBlock=TimeToBlock)

    def SeekCancel( self ):
        action = self.service.get_action("SeekCancel")
        return action.call()
            


